/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifdef __GNUC__
#pragma implementation
#endif

#include "MsgBox.h"
#include <Xm/MessageB.h>
#include <Xm/FileSB.h>
#include <Xm/MainW.h>
#include <Xm/ScrolledW.h>
#include <Xm/PanedW.h>
#include <Xm/RowColumn.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <string.h>

#include <iostream.h>

int xwMsgBox::showing = 0;

static void cb_Yes(Widget w, XtPointer client_data, XtPointer call_data)
   {int* ptr = (int*)client_data;
    ptr[0]=xwMsgBox::rsYES;
    ptr[1]=  1;
   }
static void cb_Ok(Widget w, XtPointer client_data, XtPointer call_data)
   {int* ptr = (int*)client_data;
    ptr[0]=xwMsgBox::rsOK;
    ptr[1]=  1;
   }
static void cb_No(Widget w, XtPointer client_data, XtPointer call_data)
   {int* ptr = (int*)client_data;
    ptr[0]=xwMsgBox::rsNO;
    ptr[1]= 1;
   }
static void cb_Cancel(Widget w, XtPointer client_data, XtPointer call_data)
   {int* ptr = (int*)client_data;
    ptr[0]=xwMsgBox::rsCANCEL;
    ptr[1]=     1;
   }
static void cb_Unmap(Widget w, XtPointer client_data, XtPointer call_data)
   {int* ptr = (int*)client_data;
    ptr[1]=     1;
   }


static int YESNOCANCEL(xwApp* app, const char* ptitle, const char* pmessage)
   {struct
       {int Result;
	int   Done;
       } d;
    XmString message=XmStringCreateLtoR((char*)pmessage,"tag1");
    XmString   title=XmStringCreateLocalized((char*)ptitle);

    XmString XStr_yes   =XmStringCreateLocalized((char*)"yes");
    XmString XStr_no    =XmStringCreateLocalized((char*)"no");
    XmString XStr_cancel=XmStringCreateLocalized((char*)"cancel");

    Widget dialog = XmCreateMessageDialog(app->getWindow(), (char*)ptitle, 0, 0);
    XtVaSetValues(dialog
		, XmNmessageString, message
		, XmNdialogStyle  , XmDIALOG_FULL_APPLICATION_MODAL
		, XmNokLabelString     , XStr_yes
		, XmNcancelLabelString , XStr_no
		, XmNhelpLabelString   , XStr_cancel
		, 0
		);
    XtAddCallback(dialog, XmNokCallback    , cb_Yes   , (XtPointer)&d);
    XtAddCallback(dialog, XmNcancelCallback, cb_No    , (XtPointer)&d);
    XtAddCallback(dialog, XmNhelpCallback  , cb_Cancel, (XtPointer)&d);
    XtAddCallback(dialog, XmNunmapCallback , cb_Unmap , (XtPointer)&d);

    XmStringFree(XStr_cancel);
    XmStringFree(XStr_no);
    XmStringFree(XStr_yes);

    XmStringFree(title  );
    XmStringFree(message);

    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);

    Dimension ww, wh;
    XtVaGetValues(dialog
		, XmNwidth,	&ww
		, XmNheight,	&wh
		, NULL);
    int sw = app->screenWidth();
    int sh = app->screenHeight();
    XtVaSetValues(dialog
		, XmNx,	(Position)((sw-ww)/2)
		, XmNy, (Position)((sh-wh)/2)
		, NULL);

    d.Done=0; d.Result = xwMsgBox::rsUNKNOWN;
    do {XtAppProcessEvent(app->getApp(), XtIMAll);
    } while (!d.Done);

    XtPopdown(XtParent(dialog));
    XtDestroyWidget(dialog);
    return d.Result;
   }



int xwMsgBox::ask(xwApp* app, const char* ptitle, const char* pmessage, int ptype)
   {showing = 1;
    int ret = rsUNKNOWN;
    switch (ptype)
	   {case tpYESNOCANCEL:
		 ret = YESNOCANCEL(app, ptitle, pmessage);
		 break;
	   }
    showing = 0;
    return ret;
   }


struct file_t
   {int     done;
    char*   file;
    char*    dir;
    int   button;
    Widget dialog;
   };
static void cbf_Ok(Widget w, XtPointer client_data, XtPointer call_data)
   {XmFileSelectionBoxCallbackStruct* cbs = (XmFileSelectionBoxCallbackStruct*)call_data;
    file_t* info = (file_t*)client_data;
    char*  fname;
    info->done=1;

    if (!XmStringGetLtoR(cbs->value, XmFONTLIST_DEFAULT_TAG, &fname)) return;
    if (info->file) strcpy(info->file, fname);
    info->button=(*fname)? int(xwMsgBox::rsOK) : int(xwMsgBox::rsCANCEL);
    XtFree(fname);

    if (!XmStringGetLtoR(cbs->dir, XmFONTLIST_DEFAULT_TAG, &fname)) return;
    if (info->dir) strcpy(info->dir, fname);
    XtFree(fname);
   }
static void cbf_Cancel(Widget w, XtPointer client_data, XtPointer call_data)
   {file_t* info = (file_t*)client_data;
    info->done=1;
    info->button=xwMsgBox::rsCANCEL;
   }
static void cbf_Filter(Widget w, XtPointer client_data, XtPointer call_data)
   {file_t* info = (file_t*)client_data;
    char    *ptr;
    XmString str;
    XtVaGetValues(w, XmNlabelString, &str, 0);
    XmStringGetLtoR(str, "", &ptr);
    XmStringFree(str);

    *strchr(ptr, ' ')='\0';
    str=XmStringCreateLocalized(ptr);
    XtVaSetValues(info->dialog, XmNpattern, str, 0);
    XmStringFree(str);

    XtFree(ptr);
   }
int xwMsgBox::file(xwApp* app, const char* ptitle, char* file, char* dir, const char* filetypes)
   {file_t   info;
    Widget dialog;

    showing = 1;

    dialog = XmCreateFileSelectionDialog(app->getWindow(), (char*)ptitle, NULL, 0);
    XtVaSetValues(dialog
		, XmNdialogStyle  , XmDIALOG_FULL_APPLICATION_MODAL
		, 0
		);
    if (*dir)
       {XmString str = XmStringCreateLocalized((char*)dir);
	XtVaSetValues(dialog, XmNdirectory, str, 0);
	XmStringFree(str);
       }
    XtSetSensitive(XmFileSelectionBoxGetChild(dialog, XmDIALOG_HELP_BUTTON), False);

    XtAddCallback(dialog, XmNcancelCallback, cbf_Cancel, (XtPointer)&info);
    XtAddCallback(dialog, XmNokCallback    , cbf_Ok    , (XtPointer)&info);
    XtAddCallback(dialog, XmNhelpCallback  , cbf_Cancel, (XtPointer)&info);
    XtAddCallback(dialog, XmNunmapCallback , cbf_Cancel, (XtPointer)&info);

    char filter[128];
    ((char*)memccpy(filter, filetypes, ' ', 127))[-1]='\0';
    XmString str = XmStringCreateLocalized(filter);
    XtVaSetValues(dialog, XmNpattern, str, 0);
    XmStringFree(str);

    Widget menu = XmCreatePulldownMenu(dialog, "_pulldown", NULL, 0);
    str = XmStringCreateLocalized((char*)ptitle);
    Arg args[2];
    XtSetArg(args[0], XmNsubMenuId, menu);
    XtSetArg(args[1], XmNlabelString, str);
    Widget cascade = XmCreateOptionMenu(dialog, (char*)ptitle, args, 2);
    XmStringFree(str);

    while (*filetypes)
       {char  name[128];
	const char* ptr;
	Widget   w;
	int    len;

	ptr = strchr(filetypes, '|');
	if (!ptr) ptr=filetypes+(len=strlen(filetypes));
	else {len = ptr-filetypes; ptr++; }
	strncpy(name, filetypes, len);
	name[len]='\0';
	filetypes=ptr;

	w = XtVaCreateManagedWidget(name, xmPushButtonGadgetClass, menu, 0);
	XtAddCallback(w, XmNactivateCallback, cbf_Filter, (XtPointer)&info);
       }

    XtManageChild(cascade);
    XtManageChild(dialog);


    file[0]='\0';
     dir[0]='\0';

    info.done=0;
    info.file=file;
    info.dir =dir ;
    info.button=rsUNKNOWN;
    info.dialog=dialog;
    do {XtAppProcessEvent(app->getApp(), XtIMAll);
    } while (!info.done);

    XtPopdown(XtParent(dialog));
    XtDestroyWidget(dialog);

    showing = 0;

    return info.button;
   }


